/*
 * Copyright (c) <2017> <Tuomo Lehtonen, University of Helsinki>
 * 
 * 
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * 
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */


package aba2af;

import java.util.BitSet;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 
 * @author Tuomo Lehtonen
 */
public class AndNode implements Node {

    private final List<Sentence> contents;
    private boolean isInSCC;

    private final Map<String, BitSet> argumentsContainingThis;

    public AndNode(List<Sentence> contents) {
        this.contents = contents;
        Comparator<Sentence> comparator = new Comparator<Sentence>() {
            @Override
            public int compare(Sentence s1, Sentence s2) {
                return s2.hashCode() - s1.hashCode();
            }
        };
        Collections.sort(contents, comparator);
        isInSCC = false;
        argumentsContainingThis = new HashMap<>();
    }

    // with bitsets
    public void addArgumentsContainingThis(BitSet argument) {
        String support = argument.get(0, Framework.supportSize).toString();
        if (argumentsContainingThis.containsKey(support)) {
            argumentsContainingThis.get(support).or(argument);
        } else {
            argumentsContainingThis.put(support, argument);
        }
    }

    public Collection<BitSet> getArgumentsContainingThis() {
        return argumentsContainingThis.values();
    }

    public List<Sentence> getContents() {
        return contents;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 61 * hash + Objects.hashCode(this.contents.toString());
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final AndNode other = (AndNode) obj;
        return Objects.equals(this.contents, other.contents);
    }

    @Override
    public String toString() {
        return this.contents.toString();
    }

    @Override
    public void setIsInSCC() {
        isInSCC = true;
    }

    @Override
    public boolean isInSCC() {
        return isInSCC;
    }
}
