/*
 * Copyright (c) <2017> <Tuomo Lehtonen, University of Helsinki>
 * 
 * 
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * 
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package aba2af;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.BitSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import static aba2af.Main.framework;
import static aba2af.Main.relgraphs;

/**
 *
 * @author Tuomo Lehtonen
 */
public class Printing {

    public static void AFSolverFile(BitSet[] RAs, List<List<Integer>> nonAttackLists, String filename) throws FileNotFoundException, IOException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter("afinput_" + filename + ".lp"))) {
            for (int k = 0; k < RAs.length; k++) {
                bw.append("arg(" + k + ").\n");
            }
            for (int i = 0; i < nonAttackLists.size(); i++) {
                for (Integer j : nonAttackLists.get(i)) {
                    bw.append("natt(" + i + "," + j + ").\n");
                }
            }
        }
    }

    public static void queries(String semantic, String reasoningMode, String filename, Set<Sentence> queried) throws IOException {
        Map<Sentence, List<Integer>> relevantLists = relgraphs.getArgsContainingQueriedSentences();

        for (Sentence sent : queried) {
            List<Integer> relevantList = relevantLists.get(sent);
            try (BufferedWriter wr = new BufferedWriter(
                    new FileWriter("query_" + filename + "_" + sent.getName() + ".lp"))) {
                
                wr.append("% Querying for " + reasoningMode + " acceptability of sentence " + 
                        sent.getName() + " under " + semantic + " semantics.\n");
                
                if (relevantList == null || relevantList.isEmpty()) {
                    if (semantic.equals("stb") && reasoningMode.equals("skept")) {
                        wr.append("TRUE IFF THERE IS NO STABLE ASSUMPTION SET");     // NEEDS SOMETHING
                    } else {        // If more semantics are added, this needs to be explicit, i.e. "else if"
                        wr.append("% No internally consistent argument containing queried sentence exists.\n");
                        wr.append(":-.");
                    }
                    continue;
                }
                
                if (reasoningMode.equals("cred")) {
                    if (framework.getTautologicalQueried().contains(sent)) {
                        if (semantic.equals("adm") || semantic.equals("prf")) {
                            wr.append("% Queried sentence is derivable from empty body "
                                    + "and therefore accepted\n");
                        } else if (semantic.equals("stb")) {
                            wr.append("% Accepted iff there is a stable assumption set\n");
                            continue;
                        }
                    }
                    wr.append(":- ");
                    for (int j = 0; j < relevantList.size() - 1; j++) {
                        wr.append("not in(" + relevantList.get(j) + "), ");
                    }
                    wr.append("not in(" + relevantList.get(relevantList.size() - 1) + ").");
                } else if (reasoningMode.equals("skept")) {
                    if (framework.getTautologicalQueried().contains(sent)) {
                        wr.append("% Queried sentence is derivable from empty body "
                                + "and therefore accepted\n");
                    }
                    wr.append("% True iff this query is unsatisfiable\n");
                    for (Integer arg : relevantList) {
                        wr.append(":- in(" + arg + ").\n");
                    }
                }
            }
        }
    }

    static void printTrue(String semantic, String task, Set<Sentence> tautologicalQueried, String filename) {
        for (Sentence s : tautologicalQueried) {
            try (PrintWriter pw = new PrintWriter("query_" + filename + "_" + s.getName() + ".lp")) {
                pw.println("% Querying for " + task + " acceptability of sentence " + 
                        s.getName() + " under " + semantic + " semantics.");
                pw.println("% Queried sentence is derivable from empty body "
                        + "and therefore accepted");
            } catch (FileNotFoundException ex) {
                System.err.println("Can't write to file");
                System.err.println("Error message: " + ex.getMessage());
            }
        }
    }

    static void printFalse(String semantic, String task, Set<Sentence> underivableQueried, String filename) {
        for (Sentence s : underivableQueried) {
            try (PrintWriter pw = new PrintWriter("query_" + filename + "_" + s.getName() + ".lp")) {
                pw.println("% Querying for " + task + " acceptability of sentence " + 
                        s.getName() + " under " + semantic + " semantics.");
                pw.println("% No internally consistent argument containing queried sentence exists.");
                pw.println(":-.");
            } catch (FileNotFoundException ex) {
                System.err.println("Can't write to file");
                System.err.println("Error message: " + ex.getMessage());
            }
        }
    }

    public static void resultingArguments(String outputFilename) {
        try (PrintWriter pw = new PrintWriter("arguments_" + outputFilename + ".txt")) {
            pw.println("Arguments in terms of sentences included in them:");
            for (int i = 0; i < relgraphs.getGraphs().length; i++) {
                BitSet lel = relgraphs.getGraphs()[i];
                pw.print(i + " = { ");
                for (int j = 0; j < lel.length(); j++) {
                    if (lel.get(j)) {
                        pw.print(framework.indexToSentence.get(j) + " ");
                    }
                }
                pw.println("}");
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Printing.class.getName())
                    .log(Level.SEVERE, "Can't write file \"" + outputFilename + "\"", ex);
        }
    }

    /*
     The two methods below are not in use currently. Use to visualize frameworks.
     After running these methods, run "dot -Tps <outputFileName>" and open the resulting .ps file.
     */
    public static void printAFDotFile(List<List<Integer>> nonAttackLists, String outputFilename) throws FileNotFoundException {
        try (PrintWriter wr = new PrintWriter(outputFilename)) {
            wr.println("digraph mentions {");
            for (int i = 0; i < nonAttackLists.size(); i++) {
                wr.println("\"" + i + "\""); // Actually, if there are no attacks, no need to even draw the argument. kinda
                for (int j = 0; j < nonAttackLists.get(i).size(); j++) {
                    if (nonAttackLists.get(i).get(j) == 0) {
                        wr.println("\"" + i + "\" -> \"" + j + "\"");
                    }
                }
            }
            wr.println("}");
            wr.close();
        }
    }

    public static void ruleGraphDotFile(Framework framework, String outputFilename) throws FileNotFoundException {
        /* Actual inference rules. These are gray, meaning if there is more than one outgoing gray edge from a node,
         any of those will be sufficient for deriving frm node. */
        try (PrintWriter wr = new PrintWriter(outputFilename)) {
            /* Actual inference rules. These are gray, meaning if there is more than one outgoing gray edge from a node,
             any of those will be sufficient for deriving frm node. */
            wr.println("digraph mentions {");
            for (Rule r : framework.getRules()) {
                Sentence head = r.getHead();
                AndNode body = r.getBody();
                wr.println("    \"" + head + "\" -> \"" + body + "\" [color = gray]");
            }
            // A black edge from each sentence contained in an and-node to the and-node. All of these must hold.
            for (AndNodeRule r : framework.getAndNodeDerivations()) {
                AndNode head = r.getHead();
                Sentence body = r.getBody();
                wr.println("    \"" + head + "\" -> \"" + body + "\" [color = black]");
            }
            // Assumptions are red and "tautology" is gray.
            for (Sentence s : framework.getAssumptions()) {
                wr.println("    \"" + s.getName() + "\" [style = filled, fillcolor = red]");
            }
            wr.println("    \"[truth]\" [style = filled, fillcolor = gray]");
            wr.println("    \"truth\" [style = filled, fillcolor = gray]");
            wr.println("}");
        }
    }
}
