/*
 * Copyright (c) <2017> <Tuomo Lehtonen, University of Helsinki>
 * 
 * 
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * 
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package aba2af;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

/**
 * 
 * @author Tuomo Lehtonen
 */
public class Sentence implements Node {

    private final String name;
    private Sentence contrary;
    private final boolean isAssumption;
    private final List<AndNode> children;
    
    private final Map<String, BitSet> argumentsContainingThis;  
    private boolean isSource;
    private Set<Sentence> assumptionsThisCounters;
    private boolean isInSCC;
    private final List<Integer> sentenceIndex;

    public Sentence(String name, boolean isAssumption) {
        this.name = name;
        this.isAssumption = isAssumption;
        sentenceIndex = new ArrayList<>();
        contrary = null;
        children = new ArrayList<>();
        
        argumentsContainingThis = new HashMap<>();
        
        isSource = true;
        assumptionsThisCounters = new HashSet<>();
    }

    public Set<Sentence> getAssumptionsThisCounters() {
        return assumptionsThisCounters;
    }

    public void addAssumptionThisCounters(Sentence attacks) {
        assumptionsThisCounters.add(attacks);
    }

    public boolean isSource() {
        return isSource;
    }

    public void setIsSource(boolean isSource) {
        this.isSource = isSource;
    }
    
    public void addArgumentsContainingThis(BitSet argument) {
        String support = argument.get(0, Framework.supportSize).toString();
        if (argumentsContainingThis.containsKey(support)) {
            argumentsContainingThis.get(support).or(argument);
        } else {
            argumentsContainingThis.put(support, argument);
        }
    }
    
    public Collection<BitSet> getArgumentsContainingThis() {
        return argumentsContainingThis.values();
    }

    public List<AndNode> getChildren() {
        return children;
    }

    public void addChild(AndNode supportingNode) {
        children.add(supportingNode);
    }

    public List<Integer> getSentenceIndex() {
        return sentenceIndex;
    }

    public String getName() {
        return name;
    }

    public Sentence getContrary() {
        return contrary;
    }

    public boolean isAssumption() {
        return isAssumption;
    }

    public void setContrary(Sentence contrary) {
        this.contrary = contrary;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 47 * hash + Objects.hashCode(this.name);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Sentence other = (Sentence) obj;
        return Objects.equals(this.name, other.name);
    }

    @Override
    public String toString() {
        return this.name;
    }
    
    @Override
    public void setIsInSCC() {
        this.isInSCC = true;
    }

    @Override
    public boolean isInSCC() {
        return this.isInSCC;
    }

    public void addSentenceIndex(Integer index) {
        sentenceIndex.add(index);
    }
}
